package fr.inria.phoenix.diasuite.diaswarm.generator.template.device

import fr.inria.phoenix.diasuite.diaswarm.diaswarm.ActionImpl
import fr.inria.phoenix.diasuite.diaswarm.diaswarm.AttributeDef
import fr.inria.phoenix.diasuite.diaswarm.diaswarm.DeviceType
import fr.inria.phoenix.diasuite.diaswarm.diaswarm.OrderDef
import fr.inria.phoenix.diasuite.diaswarm.diaswarm.SourceDef
import fr.inria.phoenix.diasuite.diaswarm.diaswarm.VariableDef
import fr.inria.phoenix.diasuite.diaswarm.generator.template.IConstant
import java.util.HashSet
import java.util.List
import java.util.Set

import static extension fr.inria.phoenix.diasuite.diaswarm.generator.template.datatype.DiaSwarmDatatypeHelper.*
import static extension fr.inria.phoenix.diasuite.diaswarm.generator.template.Utils.*


class DiaSwarmDeviceHelper {

	/**============================ HELPER METHODS ==============================**/
	/**
	 * Returns a string representing the filename of an device java class 
	 * along with it's namespace.
	 */
	def static generateFileName(DeviceType device) {
		device.packageName.replaceAll("\\.", "/") + "/" + "Abstract" + device.name + ".java"
	}

	def static generateFileName(SourceDef source, DeviceType device, String postfix) {
		device.packageNameForSource.replaceAll("\\.", "/") + "/" + source.name.toFirstUpper + postfix + ".java"
	}

	/**
	 * Returns a string representing the package namespace for devices.  
	 */
	def static getPackageName(DeviceType device) {
		IConstant.BASE_PACKAGE_NAMESPACE + "." + IConstant.DEVICE_PACKAGE_NAME + "." + device.name.toLowerCase
	}

	/**
	 * Returns a string representing the package namespace for devices.  
	 */
	def static getPackageNameForSource(DeviceType device) {
		IConstant.BASE_PACKAGE_NAMESPACE + "." + IConstant.DEVICE_PACKAGE_NAME + "." + device.name.toLowerCase + "." +
			IConstant.SOURCE_PACKAGE_NAME
	}

	/**
	 * Returns the full list of attributes (recursive) of a device
	 * 
	 * @param device
	 *            The device to extract attributes from
	 * @return The full list of attributes (recursive)
	 */
	def static List<AttributeDef> getAllAttributes(DeviceType device) {
		var Set<AttributeDef> processedAttributes = new HashSet<AttributeDef>();

		// Recovering attributes from the super device
		if (device.superDevice != null)
			processedAttributes.addAll(getAllAttributes(device.superDevice))

		// Recovering attributes from the device
		for (AttributeDef attribute : device.attributes)
			processedAttributes.add(attribute)

//		println("Attributes for device : " + device.name + " , attribute list : " + processedAttributes.toList.toString)
		processedAttributes.toList
	}

	/**
	 * Returns the full list of sources (recursive) of a device
	 * 
	 * @param device
	 *            The device to extract sources from
	 * @return The full list of sources (recursive)
	 */
	def static List<SourceDef> getAllSources(DeviceType device) {
		var Set<SourceDef> processedSources = new HashSet<SourceDef>()

		// Recovering sources from the super device	
		if (device.superDevice != null)
			processedSources.addAll(getAllSources(device.superDevice))

		// Recovering sources from the device 
		for (SourceDef source : device.sources)
			processedSources.add(source)

//		println("Sources for device : " + device.name + " , source list : " + processedSources.toList.toString)
		processedSources.toList
	}

	/**
	 * Returns the full list of action trigger (recursive) of a device
	 * 
	 * @param device
	 *            The device to extract actions from
	 * @return A map of order => action containing couples of orders and their containing action
	 */
	def static List<OrderDef> getAllActionMethods(DeviceType device) {
		var Set<OrderDef> actionMethods = new HashSet<OrderDef>()

		// Recovering action methods from the super device 
		if (device.superDevice != null)
			actionMethods.addAll(getAllActionMethods(device.superDevice))

		// Recovering action methods from the device 
		for (ActionImpl action : device.getActions())
			for (OrderDef method : action.getAction().getOrders())
				actionMethods.add(method)

//		println("Actions for device : " + device.name + " , action list : " + actionMethods.toList.toString)
		actionMethods.toList
	}

	/* Returns the class name for a device */
	/**
	 * 
	 */
	def static String getClassName(DeviceType device) {
		if (device.superDevice != null)
			return "/Device/" + device.superDevice.name + "/" + device.name + "/"
		return "/Device/" + device.name + "/"
	}

	/**
	 * 
	 */
	def static String getDeviceName(AttributeDef attribute) {
		(attribute.eContainer as DeviceType).name
	}

	/**
	 * 
	 */
	def static String getDeviceName(SourceDef source) {
		(source.eContainer as DeviceType).name
	}

	/**
	 * 
	 */
	def static String getVariableNames(Iterable<VariableDef> variables) {
		variables.map[name.toFirstLower].join(", ")
	}

	/**
	 * 
	 */
	def static String getParameters(Iterable<VariableDef> variables) {
		variables.map[dataTypeRef.fullName + " " + name.toFirstLower].join(",\n        ")
	}

	/**
	 * 
	 */
	def static String asString(Iterable<VariableDef> parameters) {
		transformToIndexedSet(parameters).map(['''(«value.dataTypeRef.fullName») args[«key»]''']).join(",\n        ")
	}

/*============================================================================*/
}
